package sun.security.x509;

import java.util.Locale;
import sun.security.util.DerOutputStream;
import java.io.IOException;
import sun.security.util.DerValue;

public class DNSName implements GeneralNameInterface
{
    private String name;
    private static final String alpha = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
    private static final String digitsAndHyphen = "0123456789-";
    private static final String alphaDigitsAndHyphen = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-";
    
    public DNSName(DerValue derValue) throws IOException {
        this.name = derValue.getIA5String();
    }
    
    public DNSName(String s) throws IOException {
        this(s, false);
    }
    
    public DNSName(String name, boolean b) throws IOException {
        if (name == null || name.length() == 0) {
            throw new IOException("DNS name must not be null");
        }
        if (name.indexOf(32) != -1) {
            throw new IOException("DNS names or NameConstraints with blank components are not permitted");
        }
        if (name.charAt(0) == '.' || name.charAt(name.length() - 1) == '.') {
            throw new IOException("DNS names or NameConstraints may not begin or end with a .");
        }
        int i = 0;
        while (i < name.length()) {
            int n = name.indexOf(46, i);
            if (n < 0) {
                n = name.length();
            }
            if (n - i < 1) {
                throw new IOException("DNSName SubjectAltNames with empty components are not permitted");
            }
            if (!b) {
                if ("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz".indexOf((int)name.charAt(i)) < 0) {
                    throw new IOException("DNSName components must begin with a letter");
                }
            }
            else {
                int char1 = name.charAt(i);
                if ("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz".indexOf(char1) < 0 && !Character.isDigit((char)char1)) {
                    throw new IOException("DNSName components must begin with a letter or digit");
                }
            }
            int j = i + 1;
            while (j < n) {
                if ("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-".indexOf((int)name.charAt(j)) < 0) {
                    throw new IOException("DNSName components must consist of letters, digits, and hyphens");
                }
                ++j;
            }
            i = n + 1;
        }
        this.name = name;
        return;
    }
    
    @Override
    public int getType() {
        return 2;
    }
    
    public String getName() {
        return this.name;
    }
    
    @Override
    public void encode(DerOutputStream derOutputStream) throws IOException {
        derOutputStream.putIA5String(this.name);
    }
    
    @Override
    public String toString() {
        return "DNSName: " + this.name;
    }
    
    @Override
    public boolean equals(Object o) {
        return this == o || (o instanceof DNSName && this.name.equalsIgnoreCase(((DNSName)o).name));
    }
    
    @Override
    public int hashCode() {
        return this.name.toUpperCase(Locale.ENGLISH).hashCode();
    }
    
    @Override
    public int constrains(GeneralNameInterface generalNameInterface) throws UnsupportedOperationException {
        int n;
        if (generalNameInterface == null) {
            n = -1;
        }
        else if (generalNameInterface.getType() != 2) {
            n = -1;
        }
        else {
            String lowerCase = ((DNSName)generalNameInterface).getName().toLowerCase(Locale.ENGLISH);
            String lowerCase2 = this.name.toLowerCase(Locale.ENGLISH);
            if (lowerCase.equals(lowerCase2)) {
                n = 0;
            }
            else if (lowerCase2.endsWith(lowerCase)) {
                if (lowerCase2.charAt(lowerCase2.lastIndexOf(lowerCase) - 1) == '.') {
                    n = 2;
                }
                else {
                    n = 3;
                }
            }
            else if (lowerCase.endsWith(lowerCase2)) {
                if (lowerCase.charAt(lowerCase.lastIndexOf(lowerCase2) - 1) == '.') {
                    n = 1;
                }
                else {
                    n = 3;
                }
            }
            else {
                n = 3;
            }
        }
        return n;
    }
    
    @Override
    public int subtreeDepth() throws UnsupportedOperationException {
        int n = 1;
        for (int i = this.name.indexOf(46); i >= 0; i = this.name.indexOf(46, i + 1)) {
            ++n;
        }
        return n;
    }
}
